from enum import Enum

class InstrumentType(Enum):
     """Enumerates the instruments categories in the application."""
     
     CORPORATE = "140"
     PUBLIC_BOND = "100"
     LETRAS = "110"

class Settlement(Enum):
    """Enumerates the settlement options for trades in the application."""
    
    T0 = "1" #"CI"
    T1 = "2" #"24hs"
    T2 = "3" #the same as T1 
    T3 = "4" #the same as T1
    
class OperationType(Enum):
    
    COMPRA = "10000"
    OTRO = "10040"
    
class Currency(Enum):
    
    USD = "10001"


class ApiException(Exception):
    """
    Represent a controlled exception raised by the library.
    """

    def __init__(self, msg: str):
        self.msg = msg

    def __str__(self):
        return self.msg



from typing import Any, Dict, List, Optional
import requests
import simplejson
from datetime import datetime

from . import urls
from .exceptions import ApiException


class RestClient:
    def __init__(self):
        self.session = requests.Session()
        self.messages: List[Dict[str, str]] = []

    def get_token(self, data: str, headers: dict[str,str]) -> Dict[str, Any]:
        """Makes a request to the api to get an access token

        Args:
            data (str): String with email and password
            headers (dict[str,str]): headers for the client that permits the request

        Returns:
            dict: Dict with token and user data
        """
        return self.api_request(
            urls.endpoints["token"], method="post", 
            headers=headers,
            # params=params, #it does not need it apparently
            data=data
        )
    
    def get_client_id(self, headers: dict[str,Any]) -> Dict[str,Any]:
        """Makes a request to the api to get the client_id

        Args:
            headers (dict[str,Any]): headers for the client that permits the request

        Returns:
            Str: String with client ID
        """
        return self.api_request(
            urls.endpoints["cuenta_ID"],
            headers
        )["payload"][0]["id"]
        
    def get_tickers_list(self,
        headers: dict[str,str],
        client_ID:str, 
        instrument_type:str,
        operation_type:str,
        settlement:str) -> dict[str,Any]:
        """Takes a request to get instruments quote list information filtered by instrument type, operation type and settlement
        
        Args:
            headers (dict[str,Any]): headers for the client that permits the request
            client_ID (str): string with client ID
            instrument_type (str): InstrumentType of instrument
            settlement (str): Settlement of instrument

        Returns:
            dict: Dict with the instruments information
        """
        
        return self.api_request(
            urls.endpoints["tickers_list"].format(
                client_ID, instrument_type, operation_type, settlement
                ),
            headers=headers
        )
    
    def search_tickers(self, headers: dict[str,str], short_ticker:Optional[str], item_id:Optional[str]) -> dict[str,Any]:
        """Makes a request to the api to get the information for a ticker

        Args:
            headers (dict[str,Any]): headers for the client that permits the request
            short_ticker (str): String with the short_ticker. Example: "DNC3"
            item_id (str). Example: "885981"

        Returns:
            dict: Dict with the instrument information
            
        Note:
            Should look for the short ticker or for the item_id, not both
        """
        if short_ticker and item_id:
            print("Should look for the short ticker or for the item_id, not both")
        
        search_parameter = short_ticker if short_ticker else item_id
        
        return self.api_request(
            urls.endpoints["ticker_search"].format(search_parameter),
            headers=headers
        )
    
    def get_technical_data_bonds(self, headers: dict[str,str], settlement:str, item_id:str) -> dict[str,Any]:
        """Makes a request to the api to get the technical data for bonds

        Args:
            headers (dict[str,Any]): headers for the client that permits the request
            item_id (str). Example: "885981"
            settlement (str): Settlement of instrument

        Returns:
            dict: Dict with the instrument information
            
        """
        
        return self.api_request(
            urls.endpoints["technical_data_bonds"].format(item_id, settlement),
            headers=headers
        )
    
    def get_historic_data(self, headers: dict[str,str], item_id: str, settlement: str, 
                        date_from: Optional[str] = "", date_to: Optional[str] = "") -> dict[str, Any]:
        """Makes a request to the api to get the historic data for an item

        Args:
            headers (dict[str,Any]): headers for the client that permits the request
            item_id (str). Example: "885981"
            settlement (str, optional): Settlement of instrument
            date_from (str, optional): Start date. Format yyyy-MM-dd
            date_to (str, optional): End date. Format yyyy-MM-dd
        Returns:
            dict: Dict with the instrument historic information
            
        """
        if date_from != "":
            date_from = self._get_timestamp_string(date_from)
        if date_to != "":
            date_to = self._get_timestamp_string(date_to)
        return self.api_request(
            urls.endpoints["historic_data"].format(item_id, settlement, date_from, date_to),
            headers = headers
        )
    
    def get_intraday_data(self, headers: dict[str,str], item_id: str, settlement: str, 
                        ) -> dict[str, Any]:
        """Makes a request to the api to get the intraday data for an item

        Args:
            headers (dict[str,Any]): headers for the client that permits the request
            item_id (str). Example: "885981"
            settlement (str, optional): Settlement of instrument
        Returns:
            dict: Dict with the instrument intraday information
            
        """

        return self.api_request(
            urls.endpoints["intraday_data"].format(item_id, settlement),
            headers = headers
        )
        
    def api_request(
        self,
        path: str,
        headers:dict[str,str],
        retry: bool = True,
        method: str = "get",
        params: str = "",
        json_data: Dict[str, Any] = {},
        data: str = "",
    ):
        response = None
        
        if method not in ["get", "post", "delete"]:
            raise ApiException(f"Method {method} not suported")
        if method == "get":
            response = self.session.get(self._api_url(path), headers=headers)

        if method == "post":
            response = self.session.post(
                self._api_url(path),
                params=params,
                headers = headers,
                data=data,
                json=json_data,
            )

        if method == "delete":
            response = self.session.delete(
                self._api_url(path), json=json_data, headers=headers
            )
        if not response:
            raise ApiException("Bad HTTP API Response")

        json_response = simplejson.loads(response.text)
        
        if response.status_code == 401:
            if retry:
                self.api_request(path, retry=False)
            else:
                raise ApiException("Authentication Fails.")

        if response.status_code == 500:
            raise ApiException(f"Error 500 {json_response}")

        if response.status_code == 200:
            self._log_message(path, str(json_response))
        return json_response
    
    # def update_session_headers(self, header_update: Dict[str, Any]) -> None:
    #     self.session.headers.update(header_update)
        
    def _api_url(self, path: str) -> str:
        return urls.api_url + path
    
    def _log_message(self, url: str, message: str) -> None:
        msg: Dict[str, str] = {
            "url": url,
            # "timestamp": self._get_timestamp_string(),
            "message": message,
        }
        self.messages.append(msg)
        
    def _get_timestamp_string(self, input_string) -> str:
        """
        Args:
            input_string (str, optional): Start date. Format yyyy-MM-dd
        """
        input_date = datetime.strptime(input_string, "%Y-%m-%d")
        output_string = input_date.strftime("%Y-%m-%dT%H:%M:%S.%fZ")   
        return output_string


from typing import Any, Dict, List, Optional, Tuple
import json

from py_ppi_arg.src.components import (
    RestClient,
    ApiException,
    urls,
    clientKey,
    InstrumentType,
    Settlement,
    OperationType
)


class PPI:
    def __init__(self, user: str, password: str, 
                #  gotrue_meta_security: Optional[Dict[str, Any]] = {}, api_key: Optional[str] = None
                 ) -> None:
        ## Parameters validation
        required_fields: list[tuple[str, Any, Any]]  = [
            ("user", user, str),
            ("password", password, str),
        ]
        # self._check_fields(required_fields)

        ## REST Client
        self.client: RestClient = RestClient()
        self.clientKeyheader: clientKey = clientKey().get_client_keys()

        ## Enums as instance variables
        self.instrument_types = InstrumentType
        self.settlements = Settlement
        self.operation_types = OperationType

        ## Login Information
        self.user: str = user
        self.password: str = password               
        
        self.headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:91.0) Gecko/20100101 Firefox/91.0",
            "Content-Type": "application/json",
            "Clientkey": self.clientKeyheader["ClientKey"],
            "Authorizedclient": self.clientKeyheader["AuthorizedClient"],
        }

        ## Finally, tries to authenticate
        self._auth()
    def _auth(self) -> None:
        
        """Calls the PPI API method to get an access token and updates the session headers.

        This method is responsible for authenticating the user by obtaining an access token from the PPI API.
        It then updates the session headers with the obtained token for subsequent API requests.

        Raises:
            Exception: If there is an error during the authentication process.
            Exception: If the access token is not found in the API response.
        """
        
        payload: str = json.dumps({"usuario": self.user, "clave": self.password})
        # self.client.update_session_headers(self.headers)
        # print(self.headers)
        response: Dict[str, Any] = self.client.get_token(data=payload, headers=self.headers)
        # print(response)
        if response["status"] != 0:
            raise Exception(f'Error: {response["message"]}')

        if "accessToken" not in response["payload"]["token"]:
            raise Exception("Error: Access token not found in the API response")

        self.access_token = response["payload"]["token"]["accessToken"]
        self.clientkey = response["payload"]["token"]["clienteID"]
        self._auth_phase_2()
        self.clientID = self.client.get_client_id(self.headers)
        
    def _auth_phase_2(self) -> None:
        
        """Updates the session headers and performs additional steps after login.

        After successful login and obtaining the access token, this method is responsible for updating the session headers
        with the necessary authentication information. It replicates the workflow of the web app.

        """
        
        headers_update: dict[str, str] = {
            "authorization": f"bearer {self.access_token}",
        }
        # print(headers_update)
        self.headers.update(headers_update)
    
    def get_tickers_list(self,
        instrument_type: InstrumentType,
        operation_type: OperationType,
        settlement: Settlement) -> dict[str,Any]:
        
        """Takes a request to get instruments quote list information filtered by instrument type, operation type and settlement
        
        Args:
            client_ID (str): string with client ID
            instrument_type (str): InstrumentType of instrument
            operation_type (str): OperationType for instrument
            settlement (str): Settlement for instrument

        Returns:
            dict: Dict with the instruments information
        """
        
        return self.client.get_tickers_list(
            self.headers, self.clientID, instrument_type.value, operation_type.value, settlement.value
        )
                
    def search_tickers(self, short_ticker: Optional[str] = None, item_id: Optional[str] = None) -> dict[str,any]:
        
        """Makes a request to the api to get the information for a ticker

        Args:
            headers (dict[str,Any]): headers for the client that permits the request
            short_ticker (str): String with the short_ticker. Example: "DNC3"
            item_id (str). Example: "885981"

        Returns:
            dict: Dict with the instrument information
            
        Note:
            Should look for the short ticker or for the item_id, not both
        """
        return self.client.search_tickers(
            headers = self.headers,
            short_ticker=short_ticker,
            item_id=item_id
        )
        
    def get_technical_data_bonds(self, settlement: Settlement, item_id: str):
        
        """Takes a request to get techical data for a bond
        
        Args:
            settlement (str): Settlement for instrument
            item_id (str). Example: "885981"

        Returns:
            dict: Dict with the instruments information
        """
        
        return self.client.get_technical_data_bonds(self.headers, settlement.value, item_id)
    
    def get_historic_data(self, item_id: str, settlement: Settlement, 
                        date_from: Optional[str] = "", date_to: Optional[str] = "") -> dict[str, Any]:
        """Makes a request to the api to get the historic data for an item

        Args:
            headers (dict[str,Any]): headers for the client that permits the request
            item_id (str). Example: "885981"
            settlement (str, optional): Settlement of instrument
            date_from (str, optional): Start date. Format yyyy-MM-dd
            date_to (str, optional): End date. Format yyyy-MM-dd
        Returns:
            dict: Dict with the instrument historic information
            
        """

        return self.client.get_historic_data(self.headers, item_id, settlement.value, date_from, date_to)
    
    def get_intraday_data(self, item_id: str, settlement: Settlement, 
                        ) -> dict[str, Any]:
        """Makes a request to the api to get the intraday data for an item

        Args:
            headers (dict[str,Any]): headers for the client that permits the request
            item_id (str). Example: "885981"
            settlement (str, optional): Settlement of instrument
        Returns:
            dict: Dict with the instrument intraday information
            
        """

        return self.client.get_intraday_data(self.headers, item_id, settlement.value)


# -*- coding: utf-8 -*-
"""
pyPPI.urls
Defines all API Paths
"""

api_url = "https://api.portfoliopersonal.com/"
account_url = 'https://cuenta.portfoliopersonal.com'

endpoints = {
    "token": "api/Seguridad/Auth/Login",
    "cuenta_ID": "/api/Cuenta/ComitentesAsignados",
    "refresh_token": "/api/Seguridad/Auth/RefreshToken",
    "ticker_search": "/api/Cotizaciones/Item/Search?q={}", #short ticker or id
    "ranking": "/api/Cotify/Ranking",
    "bonds_groups":"/api/Cotizaciones/Bono/PorGrupo?id=13463",
    "tickers_list": "/api/Ordenes/InstrumentosOperables?cuentaID={}&tipoProducto={}&tipoOperacion={}&tipoPlazoLiquidacion={}",
    "technical_data_bonds": "/api/Cotizaciones/Bono/{}/DatosTecnicos?plazoLiquidacionID={}",
    "historic_data":"/api/Cotizaciones/Item/{}/Historico/{}?fechaDesde={}&fechaHasta={}",
    "intraday_data":"/api/Cotizaciones/Item/{}/Intradiario?idPlazo={}"
}




import requests
from bs4 import BeautifulSoup
import re
import py_ppi_arg.src.components.urls as urls

class clientKey:
    def __init__(self):
        self.url = urls.account_url
        self.js_files = self.fetch_js_files()

    def fetch_js_files(self):
        response = requests.get(self.url)
        if response.status_code != 200:
            print(f"Failed to fetch the webpage: {response.status_code}")
            return []
        soup = BeautifulSoup(response.text, 'html.parser')
        script_tags = soup.find_all('script')
        pattern = re.compile(r'/_next/static/chunks/.*\.js')
        js_files = [tag['src'] for tag in script_tags if 'src' in tag.attrs and pattern.match(tag['src'])]
        return js_files

    def fetch_js_file_content(self, js_file_url):
        response = requests.get(self.url + js_file_url)
        if response.status_code != 200:
            print(f"Failed to fetch the JavaScript file: {response.status_code}")
        return response

    def find_client_key_function(self, js_file_content):
        pattern = re.compile(r'function\(\w+,\s*\w+,\s*\w+\)\s*\{[^}]*ClientKey:\s*\w+[^}]*\}')
        match = pattern.search(js_file_content.text)
        if match:
            return match.group()
        # print("Could not find the function where ClientKey is defined")
        return None

    def extract_client_keys(self, function_text):
        pattern = re.compile(r'let\s+s="(\d+)",\s*r="(\w+)";')
        match = pattern.search(function_text)
        if match:
            authorized_client = match.group(1)
            client_key = match.group(2)
            return {"AuthorizedClient": authorized_client, "ClientKey": client_key}
        else:
            return {"AuthorizedClient": None, "ClientKey": None}

    def get_client_keys(self):
        for js_file in self.js_files:
            js_file_content = self.fetch_js_file_content(js_file)
            if js_file_content:
                function_text = self.find_client_key_function(js_file_content)
                if function_text:
                    client_keys = self.extract_client_keys(function_text)
                    return client_keys
        print("Could not find the function where ClientKey is defined")